@@ -5,13 +5,14 @@ from thunder.factorization.pca import PCA
 from thunder.factorization.svd import SVD
 from thunder.factorization.ica import ICA
 from thunder.factorization.nmf import NMF
-from thunder.regression.regress import RegressionModel
-from thunder.regression.regress import MeanRegressionModel, LinearRegressionModel, BilinearRegressionModel
 from thunder.clustering.kmeans import KMeans, KMeansModel
-from thunder.regression.tuning import TuningModel
-from thunder.regression.tuning import GaussianTuningModel, CircularTuningModel
-from thunder.imgprocessing.registration import Registration, RegistrationModel
-from thunder.imgprocessing.regmethods.crosscorr import CrossCorr, PlanarCrossCorr
+from thunder.regression.linear.algorithms import LinearRegression 
+from thunder.regression.linear.algorithms import OrdinaryLinearRegression, TikhonovLinearRegression, RidgeLinearRegression
+from thunder.regression.nonlinear.tuning import TuningModel
+from thunder.regression.nonlinear.tuning import GaussianTuningModel, CircularTuningModel 
+from thunder.registration.registration import Registration, RegistrationModel
+from thunder.extraction.extraction import SourceExtraction
+from thunder.extraction.source import Source, SourceModel
 
 # data types
 from thunder.rdds.series import Series
@@ -25,4 +26,4 @@ from thunder.viz.colorize import Colorize
 from thunder.utils.datasets import DataSets
 from thunder.utils.context import ThunderContext
 
-__version__ = "0.5.1"
+__version__ = "0.6.0"
\ No newline at end of file
