@@ -1,4 +1,4 @@
-from numpy import asarray, maximum, minimum
+from numpy import asarray, maximum, minimum, add
 
 
 class Data(object):
@@ -20,12 +20,13 @@ class Data(object):
         directly exposed by the Data object can be accessed via `obj.rdd`.
     """
 
-    _metadata = ['_nrecords', '_dtype']
+    _metadata = ['_nrecords', '_dtype', '_shape']
 
     def __init__(self, rdd, nrecords=None, dtype=None):
         self.rdd = rdd
         self._nrecords = nrecords
         self._dtype = dtype
+        self._shape = None
 
     def __repr__(self):
         # start with class name
@@ -112,7 +113,8 @@ class Data(object):
 
     def _checkOverwrite(self, outputDirPath):
         """ Checks for existence of outputDirPath, raising ValueError if it already exists """
-        from thunder.utils.common import AWSCredentials, raiseErrorIfPathExists
+        from thunder.utils.aws import AWSCredentials
+        from thunder.utils.common import raiseErrorIfPathExists
         awsCredentialOverride = AWSCredentials.fromContext(self.rdd.ctx)
         raiseErrorIfPathExists(outputDirPath, awsCredentialsOverride=awsCredentialOverride)
 
@@ -152,10 +154,10 @@ class Data(object):
 
     def get(self, key):
         """
-        Returns a single value matching the passed key, or None if no matching keys found.
+        Returns single value locally to driver that matches the passed key
 
         If multiple records are found with keys matching the passed key, a sequence of all matching
-        values will be returned.
+        values will be returned. If no records found, will return None
         """
         firstKey = self.first()[0]
         Data.__getKeyTypeCheck(firstKey, key)
@@ -169,7 +171,7 @@ class Data(object):
 
     def getMany(self, keys):
         """
-        Returns a sequence of values corresponding to the passed sequence of keys.
+        Returns values locally to driver that correspond to the passed sequence of keys.
 
         The return value will be a sequence equal in length to the passed keys, with each
         value in the returned sequence corresponding to the key at the same position in the passed
@@ -195,7 +197,7 @@ class Data(object):
 
     def getRange(self, sliceOrSlices, keys=True):
         """
-        Returns key/value pairs that fall within a range given by the passed slice or slices.
+        Returns key/value pairs locally to driver that fall within a given range.
 
         The return values will be a sorted list of key/value pairs of all records in the underlying
         RDD for which the key falls within the range given by the passed slice selectors. Note that
@@ -372,7 +374,7 @@ class Data(object):
 
         See also
         --------
-        Series.apply
+        Data.apply
         """
 
         return self.apply(lambda (k, v): (func(k), v), **kwargs)
@@ -383,13 +385,13 @@ class Data(object):
 
         See also
         --------
-        Series.apply
+        Data.apply
         """
         return self.apply(lambda (k, v): (k, func(v)), **kwargs)
 
     def collect(self, sorting=False):
         """
-        Return all records to the driver
+        Return all records locally to the driver
 
         This will be slow for large datasets, and may exhaust the available memory on the driver.
 
@@ -402,7 +404,7 @@ class Data(object):
 
     def collectAsArray(self, sorting=False):
         """
-        Return all keys and values to the driver as a tuple of numpy arrays
+        Return all keys and values locally to the driver as a tuple of numpy arrays
 
         This will be slow for large datasets, and may exhaust the available memory on the driver.
         """
@@ -425,7 +427,7 @@ class Data(object):
 
     def collectKeysAsArray(self, sorting=False):
         """
-        Return all values to the driver as a numpy array
+        Return all values locally to the driver as a numpy array
 
         This will be slow for large datasets, and may exhaust the available memory on the driver.
         """
@@ -478,7 +480,7 @@ class Data(object):
         obj.sum() is equivalent to obj.astype(dtype, casting).rdd.values().sum().
         """
         out = self.astype(dtype, casting)
-        return out.rdd.values().sum()
+        return out.rdd.values().treeReduce(add, depth=3)
 
     def variance(self, dtype='float64', casting='safe'):
         """
@@ -526,17 +528,19 @@ class Data(object):
             return left_counter.mergeStats(right_counter)
 
         out = self.astype(dtype, casting)
-        return out.values().mapPartitions(lambda i: [StatCounter(i, stats=requestedStats)]).reduce(redFunc)
+        parts = out.values().mapPartitions(lambda i: [StatCounter(i, stats=requestedStats)])
+        result = parts.treeReduce(redFunc, depth=3)
+        return result
 
     def max(self):
         """ Maximum of values across keys, returned as an ndarray. """
         # NOTE: Does not use stats('max') to prevent cast to float64
-        return self.rdd.values().reduce(maximum)
+        return self.rdd.values().treeReduce(maximum, depth=3)
 
     def min(self):
         """ Minimum of values across keys, returned as an ndarray. """
         # NOTE: Does not use stats('min') to prevent cast to float64
-        return self.rdd.values().reduce(minimum)
+        return self.rdd.values().treeReduce(minimum, depth=3)
 
     def coalesce(self, numPartitions):
         """
@@ -584,13 +588,46 @@ class Data(object):
         Filter records by applying a function to each record.
 
         This calls the Spark filter() method on the underlying RDD.
+
+        Parameters
+        ----------
+        func : function
+            The function to compute on each record, should evaluate to Boolean
         """
         return self._constructor(self.rdd.filter(lambda d: func(d))).__finalize__(self)._resetCounts()
 
     def filterOnKeys(self, func):
-        """ Filter records by applying a function to keys """
+        """
+        Filter records by applying a function to keys
+
+        See also
+        --------
+        Data.filter
+        """
         return self._constructor(self.rdd.filter(lambda (k, v): func(k))).__finalize__(self)._resetCounts()
 
     def filterOnValues(self, func):
-        """ Filter records by applying a function to values """
+        """
+        Filter records by applying a function to values
+
+        See also
+        --------
+        Data.filter
+        """
         return self._constructor(self.rdd.filter(lambda (k, v): func(v))).__finalize__(self)._resetCounts()
+
+    def range(self, start, stop):
+        """
+        Extract records with keys that fall inside a range
+
+        Returns another Data object, unlike
+        getRange which returns a local array to the driver
+
+        See also
+        --------
+        Data.filterOnKeys
+        """
+        if start >= stop:
+            raise ValueError("Start (%g) is greater than or equal to stop (%g), result will be empty" % (start, stop))
+
+        return self.filterOnKeys(lambda k: start <= k < stop)
\ No newline at end of file
