@@ -23,6 +23,12 @@ class Images(Data):
                 raise TypeError("Images dims parameter must be castable to Dimensions object, got: %s" % str(dims))
         self._dims = dims
 
+    @property
+    def shape(self):
+        if self._shape is None:
+            self._shape = (self.nrecords,) + self.dims.count
+        return self._shape
+
     @property
     def dims(self):
         if self._dims is None:
@@ -219,7 +225,7 @@ class Images(Data):
         from matplotlib.pyplot import imsave
         from io import BytesIO
         from thunder.rdds.fileio.writers import getParallelWriterForPath, getCollectedFileWriterForPath
-        from thunder.utils.common import AWSCredentials
+        from thunder.utils.aws import AWSCredentials
 
         def toFilenameAndPngBuf(kv):
             key, img = kv
@@ -260,14 +266,19 @@ class Images(Data):
         """
         from thunder.rdds.fileio.writers import getParallelWriterForPath
         from thunder.rdds.fileio.imagesloader import writeBinaryImagesConfig
-        from thunder.utils.common import AWSCredentials
+        from thunder.utils.aws import AWSCredentials
+        import StringIO
 
         dimsTotal = list(asarray(self.dims.max)-asarray(self.dims.min)+1)
 
         def toFilenameAndBinaryBuf(kv):
             key, img = kv
             fname = prefix+"-"+"%05d.bin" % int(key)
-            return fname, img.transpose().copy()
+            buf = StringIO.StringIO()
+            buf.write(img.transpose().copy().tostring())
+            val = buf.getvalue()
+            buf.close()
+            return fname, val
 
         bufRdd = self.rdd.map(toFilenameAndBinaryBuf)
 
@@ -445,6 +456,43 @@ class Images(Data):
         return self._constructor(
             self.rdd.mapValues(lambda v: filter_(v))).__finalize__(self)
 
+    def localCorr(self, neighborhood=2):
+        """
+        Correlate every pixel to the average of its local neighborhood.
+
+        This algorithm computes, for every spatial record, the correlation coefficient
+        between that record's series, and the average series of all records within
+        a local neighborhood with a size defined by the neighborhood parameter.
+        The neighborhood is currently required to be a single integer, which represents the neighborhood
+        size in both x and y.
+
+        parameters
+        ----------
+        neighborhood: int, optional, default=2
+            Size of the correlation neighborhood (in both the x and y directions), in pixels.
+        """
+
+        if not isinstance(neighborhood, int):
+            raise ValueError("The neighborhood must be specified as an integer.")
+
+        from numpy import corrcoef
+
+        nimages = self.nrecords
+
+        # Spatially average the original image set over the specified neighborhood
+        blurred = self.uniformFilter((neighborhood * 2) + 1)
+
+        # Union the averaged images with the originals to create an Images object containing 2N images (where
+        # N is the original number of images), ordered such that the first N images are the averaged ones.
+        combined = self.rdd.union(blurred.applyKeys(lambda k: k + nimages).rdd)
+        combinedImages = self._constructor(combined, nrecords=(2 * nimages)).__finalize__(self)
+
+        # Correlate the first N (averaged) records with the last N (original) records
+        series = combinedImages.toSeries()
+        corr = series.applyValues(lambda x: corrcoef(x[:nimages], x[nimages:])[0, 1])
+
+        return corr.pack()
+
     def crop(self, minbound, maxbound):
         """
         Crop a spatial region from 2D or 3D data.
@@ -593,26 +641,33 @@ class Images(Data):
         data = self.rdd.map(selectFcn)
         return self._constructor(data, dims=(1, nregions)).__finalize__(self)
 
-    def planes(self, startidz, stopidz):
+    def planes(self, startidz, stopidz=None):
         """
         Subselect planes from 3D image data.
 
         Parameters
         ----------
         startidz, stopidz : int
-            Indices of region to crop in z, interpreted according to python slice indexing conventions.
+            Range of planes to include. Can either provide a start and stop plane,
+            or provide a single list.
 
         See also
         --------
         Images.crop
         """
-
         dims = self.dims
 
         if len(dims) == 2 or dims[2] == 1:
             raise Exception("Cannot subselect planes, images must be 3D")
 
-        return self.crop([0, 0, startidz], [dims[0], dims[1], stopidz])
+        if stopidz is not None:
+            return self.crop([0, 0, startidz], [dims[0], dims[1], stopidz])
+        else:
+            if max(startidz) > dims[2]-1:
+                raise ValueError('maxmium selected plane %g exceeds size %g' % (max(startidz), dims[2]))
+            newrdd = self.rdd.mapValues(lambda v: v[:, :, startidz])
+            newdims = (dims[0], dims[1], len(startidz))
+            return self._constructor(newrdd, dims=newdims).__finalize__(self)
 
     def subtract(self, val):
         """
