@@ -55,6 +55,12 @@ class Series(Data):
                 raise TypeError("Series dims parameter must be castable to Dimensions object, got: %s" % str(dims))
         self._dims = dims
 
+    @property
+    def shape(self):
+        if self._shape is None:
+            self._shape = tuple(self.dims) + (size(self.index),)
+        return self._shape
+
     @property
     def index(self):
         if self._index is None:
@@ -398,6 +404,59 @@ class Series(Data):
         dims.max = list(array(dims.max)[arange(0, nkeys) != axis])
         return self._constructor(rdd, dims=dims).__finalize__(self)
 
+    def _checkFixedLength(self, length):
+        """
+        Check that given fixed length evenly divides index
+
+        Parameters
+        ----------
+        length : int
+            Fixed length with which to subdivide index
+        """
+        n = len(self.index)
+        if divmod(n, length)[1] != 0:
+            raise ValueError('Fixed length, %g, must evenly divide length of series, %g'
+                             % (length, n))
+        if n == length:
+            raise ValueError('Length, %g, cannot be length of series, %g'
+                             % (length, n))
+
+    def meanByFixedLength(self, length):
+        """
+        Compute the mean across fixed length portions of each record
+
+        Parameters
+        ----------
+        length : int
+            Fixed length with which to subdivide
+        """
+        self._checkFixedLength(length)
+        func = lambda v: v.reshape(-1, length).mean(axis=0)
+        rdd = self.rdd.mapValues(func)
+        index = arange(0, length)
+        return self._constructor(rdd, index=index).__finalize__(self)
+
+    def groupByFixedLength(self, length):
+        """
+        Regroup each record by subdividing into fixed length portions
+
+        Will yield a new Series with N times as many records
+        as the initial Series, where N is the number of chunks
+        of fixed length.
+
+        Parameters
+        ----------
+        length : int
+            Fixed length with which to subdivide
+        """
+        self._checkFixedLength(length)
+        n = len(self.index) / length
+        func = lambda (k, v): zip([k + (i,) for i in range(0, n)], list(v.reshape(-1, length)))
+        rdd = self.rdd.flatMap(func)
+        index = arange(0, length)
+        count = self.nrecords * n
+        return self._constructor(rdd, index=index, nrecords=count).__finalize__(self)
+
     def subToInd(self, order='F', isOneBased=True):
         """
         Convert subscript index keys to linear index keys
@@ -863,7 +922,7 @@ class Series(Data):
         from thunder.rdds.imgblocks.blocks import SimpleBlocks
         from thunder.rdds.fileio.writers import getParallelWriterForPath
         from thunder.rdds.fileio.seriesloader import writeSeriesConfig
-        from thunder.utils.common import AWSCredentials
+        from thunder.utils.aws import AWSCredentials
 
         if not overwrite:
             self._checkOverwrite(outputDirPath)
@@ -951,7 +1010,7 @@ class Series(Data):
         lenIdx = index.shape[0]
         nlevels = index.shape[1]
 
-        combs = product(*[unique(index.T[i,:]) for i in xrange(nlevels)])
+        combs = product(*[unique(index.T[i, :]) for i in xrange(nlevels)])
         combs = array([l for l in combs])
 
         masks = array([[array_equal(index[i], c) for i in xrange(lenIdx)] for c in combs])
@@ -978,9 +1037,9 @@ class Series(Data):
         index = array(ind)
         if len(index[0]) == 1:
             index = ravel(index)
-        return Series(newrdd, index=index).__finalize__(self, noPropagate=('_dtype',))
+        return self._constructor(newrdd, index=index).__finalize__(self, noPropagate=('_dtype',))
 
-    def selectByIndex(self, val, level=0, squeeze=False, filter=False):
+    def selectByIndex(self, val, level=0, squeeze=False, filter=False, returnMask=False):
         """
         Select or filter elements of the Series by index values (across levels, if multi-index).
 
@@ -1013,6 +1072,9 @@ class Series(Data):
         filter: bool, optional, default=False
             If True, selection process is reversed and all index values EXCEPT those specified
             are selected.
+
+        returnMask: bool, optional, default=False
+            If True, return the mask used to implement the selection.
         """
 
         try:
@@ -1073,7 +1135,12 @@ class Series(Data):
         elif len(indFinal[1]) == 0:
             indFinal = arange(sum(finalMask))
 
-        return Series(newrdd, index=indFinal).__finalize__(self)
+        result = self._constructor(newrdd, index=indFinal).__finalize__(self)
+
+        if returnMask:
+            return result, finalMask
+        else:
+            return result
 
     def seriesAggregateByIndex(self, function, level=0):
         """
