@@ -13,15 +13,6 @@ class ThunderContext():
 
     Also supports creation of example datasets, and loading example
     data both locally and from EC2.
-
-    Attributes
-    ----------
-    `_sc` : SparkContext
-        Spark context for Spark functionality
-
-    `_credentials` : AWSCredentials object, optional, default = None
-        Stores public and private keys for AWS services. Typically available through
-        configuration files, and but can optionally be set using :func:`ThunderContext.setAWSCredentials()`.
     """
     def __init__(self, sparkcontext):
         self._sc = sparkcontext
@@ -35,8 +26,29 @@ class ThunderContext():
         from pyspark import SparkContext
         return ThunderContext(SparkContext(*args, **kwargs))
 
+    def addPyFile(self, path):
+        """
+        Adds a .zip or .py or .egg dependency for all tasks to be executed
+        as part of this context.
+
+        Uses the corresponding SparkContext method.
+
+        Parameters
+        ----------
+        path : str
+            Path to a file as either a local file, file in HDFS, or URI.
+        """
+        self._sc.addPyFile(path)
+        
+    def stop(self):
+        """
+        Shut down the context
+        """
+        self._sc.stop()
+
     def loadSeries(self, dataPath, nkeys=None, nvalues=None, inputFormat='binary', minPartitions=None,
-                   confFilename='conf.json', keyType=None, valueType=None, keyPath=None, varName=None):
+                   maxPartitionSize='32mb', confFilename='conf.json', keyType=None, valueType=None, keyPath=None,
+                   varName=None):
         """
         Loads a Series object from data stored as binary, text, npy, or mat.
 
@@ -64,6 +76,10 @@ class ThunderContext():
         minPartitions: int, optional, default = SparkContext.minParallelism
             Minimum number of Spark partitions to use, only for text.
 
+        maxPartitionSize : int, optional, default = '32mb'
+            Maximum size of partitions as a Java-style memory string, e.g. '32mb' or '64mb',
+            indirectly controls the number of Spark partitions, only for binary.
+
         confFilename: string, optional, default 'conf.json'
             Path to JSON file with configuration options including 'nkeys', 'nvalues',
             'keyType', and 'valueType'. If a file is not found at the given path, then the base
@@ -93,7 +109,7 @@ class ThunderContext():
 
         if inputFormat.lower() == 'binary':
             data = loader.fromBinary(dataPath, confFilename=confFilename, nkeys=nkeys, nvalues=nvalues,
-                                     keyType=keyType, valueType=valueType)
+                                     keyType=keyType, valueType=valueType, maxPartitionSize=maxPartitionSize)
         elif inputFormat.lower() == 'text':
             if nkeys is None:
                 raise Exception('Must provide number of keys per record for loading from text')
@@ -114,7 +130,8 @@ class ThunderContext():
         Loads an Images object from data stored as a binary image stack, tif, or png files.
 
         Supports single files or multiple files, stored on a local file system, a networked file sytem
-        (mounted and available on all nodes), or Amazon S3. HDFS is not currently supported for image file data.
+        (mounted and available on all nodes), Amazon S3, or Google Storage.
+        HDFS is not currently supported for image file data.
 
         Parameters
         ----------
@@ -162,7 +179,8 @@ class ThunderContext():
             to True to ensure consistent keys.
 
         npartitions: positive int, optional, default = None
-            Specify number of partitions for the RDD, if unspecified will use 1 partition per image.
+            Specify number of partitions for the RDD, if unspecified will use as many partitions
+            as available cores
 
         renumber: boolean, optional, default = False
             Recalculate keys for records after images are loading. Only necessary if different files contain
@@ -182,6 +200,9 @@ class ThunderContext():
         from thunder.rdds.fileio.imagesloader import ImagesLoader
         loader = ImagesLoader(self._sc)
 
+        if npartitions is None:
+            npartitions = self._sc.defaultParallelism
+
         # Checking StartIdx is smaller or equal to StopIdx
         if startIdx is not None and stopIdx is not None and startIdx > stopIdx:
             raise Exception("Error. startIdx {} is larger than stopIdx {}".inputFormat(startIdx, stopIdx))
@@ -207,6 +228,72 @@ class ThunderContext():
         else:
             return data.renumber()
 
+    def loadSeriesFromArray(self, values, index=None, npartitions=None):
+        """
+        Load Series data from a local array
+
+        Parameters
+        ----------
+        values : list or ndarray
+            A list of 1d numpy arrays, or a single 2d numpy array
+
+        index : array-like, optional, deafult = None
+            Index to set for Series object, if None will use linear indices.
+
+        npartitions : position int, optional, default = None
+            Number of partitions for RDD, if unspecified will use
+            default parallelism.
+        """
+        from numpy import ndarray, asarray
+        from thunder.rdds.fileio.seriesloader import SeriesLoader
+        loader = SeriesLoader(self._sc)
+
+        if not npartitions:
+            npartitions = self._sc.defaultParallelism
+
+        if isinstance(values, list):
+            values = asarray(values)
+
+        if isinstance(values, ndarray) and values.ndim > 1:
+            values = list(values)
+
+        data = loader.fromArrays(values, npartitions=npartitions)
+
+        if index:
+            data.index = index
+
+        return data
+
+    def loadImagesFromArray(self, values, npartitions=None):
+        """
+        Load Images data from a local array
+
+        Parameters
+        ----------
+        values : list or ndarray
+            A list of 2d or 3d numpy arrays,
+            or a single 3d or 4d numpy array
+
+        npartitions : position int, optional, default = None
+            Number of partitions for RDD, if unspecified will use
+            default parallelism.
+        """
+        from numpy import ndarray, asarray
+
+        from thunder.rdds.fileio.imagesloader import ImagesLoader
+        loader = ImagesLoader(self._sc)
+
+        if isinstance(values, list):
+            values = asarray(values)
+
+        if isinstance(values, ndarray) and values.ndim > 2:
+            values = list(values)
+
+        if not npartitions:
+            npartitions = self._sc.defaultParallelism
+
+        return loader.fromArrays(values, npartitions=npartitions)
+
     def loadImagesOCP(self, bucketName, resolution, server='ocp.me', startIdx=None, stopIdx=None,
                       minBound=None, maxBound=None):
         """
@@ -256,7 +343,7 @@ class ThunderContext():
 
     def loadImagesAsSeries(self, dataPath, dims=None, inputFormat='stack', ext=None, dtype='int16',
                            blockSize="150M", blockSizeUnits="pixels", startIdx=None, stopIdx=None,
-                           shuffle=True, recursive=False, nplanes=None, npartitions=None,
+                           recursive=False, nplanes=None, npartitions=None,
                            renumber=False, confFilename='conf.json'):
         """
         Load Images data as Series data.
@@ -304,9 +391,6 @@ class ThunderContext():
         stopIdx: nonnegative int, optional, default = None
             See startIdx.
 
-        shuffle: boolean, optional, default = True
-            Controls whether the conversion from Images to Series formats will use of a Spark shuffle-based method.
-
         recursive: boolean, optional, default = False
             If true, will recursively descend directories rooted at dataPath, loading all files
             in the tree with an appropriate extension.
@@ -341,42 +425,23 @@ class ThunderContext():
         if not ext:
             ext = DEFAULT_EXTENSIONS.get(inputFormat.lower(), None)
 
-        if shuffle:
-            from thunder.rdds.fileio.imagesloader import ImagesLoader
-            loader = ImagesLoader(self._sc)
-            if inputFormat.lower() == 'stack':
-                images = loader.fromStack(dataPath, dims, dtype=dtype, ext=ext, startIdx=startIdx, stopIdx=stopIdx,
-                                          recursive=recursive, nplanes=nplanes, npartitions=npartitions,
-                                          confFilename=confFilename)
-            else:
-                # tif / tif stack
-                images = loader.fromTif(dataPath, ext=ext, startIdx=startIdx, stopIdx=stopIdx,
-                                        recursive=recursive, nplanes=nplanes, npartitions=npartitions)
-            if renumber:
-                images = images.renumber()
-            return images.toBlocks(blockSize, units=blockSizeUnits).toSeries()
-
+        from thunder.rdds.fileio.imagesloader import ImagesLoader
+        loader = ImagesLoader(self._sc)
+        if inputFormat.lower() == 'stack':
+            images = loader.fromStack(dataPath, dims, dtype=dtype, ext=ext, startIdx=startIdx, stopIdx=stopIdx,
+                                      recursive=recursive, nplanes=nplanes, npartitions=npartitions,
+                                      confFilename=confFilename)
         else:
-            from thunder.rdds.fileio.seriesloader import SeriesLoader
-            if nplanes is not None:
-                raise NotImplementedError("nplanes is not supported with shuffle=False")
-            if npartitions is not None:
-                raise NotImplementedError("npartitions is not supported with shuffle=False")
-            if renumber:
-                raise NotImplementedError("renumber is not supported with shuffle=False")
-
-            loader = SeriesLoader(self._sc)
-            if inputFormat.lower() == 'stack':
-                return loader.fromStack(dataPath, dims, ext=ext, dtype=dtype, blockSize=blockSize,
-                                        startIdx=startIdx, stopIdx=stopIdx, recursive=recursive)
-            else:
-                # tif / tif stack
-                return loader.fromTif(dataPath, ext=ext, blockSize=blockSize,
-                                      startIdx=startIdx, stopIdx=stopIdx, recursive=recursive)
+            # tif / tif stack
+            images = loader.fromTif(dataPath, ext=ext, startIdx=startIdx, stopIdx=stopIdx,
+                                    recursive=recursive, nplanes=nplanes, npartitions=npartitions)
+        if renumber:
+            images = images.renumber()
+        return images.toBlocks(blockSize, units=blockSizeUnits).toSeries()
 
     def convertImagesToSeries(self, dataPath, outputDirPath, dims=None, inputFormat='stack', ext=None,
                               dtype='int16', blockSize="150M", blockSizeUnits="pixels", startIdx=None, stopIdx=None,
-                              shuffle=True, overwrite=False, recursive=False, nplanes=None, npartitions=None,
+                              overwrite=False, recursive=False, nplanes=None, npartitions=None,
                               renumber=False, confFilename='conf.json'):
         """
         Write out Images data as Series data, saved in a flat binary format.
@@ -434,9 +499,6 @@ class ThunderContext():
         stopIdx: nonnegative int, optional, default = None
             See startIdx.
 
-        shuffle: boolean, optional, default = True
-            Controls whether the conversion from Images to Series formats will use of a Spark shuffle-based method.
-
         overwrite: boolean, optional, default False
             If true, the directory specified by outputDirPath will be deleted (recursively) if it
             already exists. (Use with caution.)
@@ -471,42 +533,25 @@ class ThunderContext():
         if not ext:
             ext = DEFAULT_EXTENSIONS.get(inputFormat.lower(), None)
 
-        if shuffle:
-            from thunder.rdds.fileio.imagesloader import ImagesLoader
-            loader = ImagesLoader(self._sc)
-            if inputFormat.lower() == 'stack':
-                images = loader.fromStack(dataPath, dims, ext=ext, dtype=dtype, startIdx=startIdx, stopIdx=stopIdx,
-                                          recursive=recursive, nplanes=nplanes, npartitions=npartitions,
-                                          confFilename=confFilename)
-            else:
-                # 'tif' or 'tif-stack'
-                images = loader.fromTif(dataPath, ext=ext, startIdx=startIdx, stopIdx=stopIdx,
-                                        recursive=recursive, nplanes=nplanes, npartitions=npartitions)
-            if renumber:
-                images = images.renumber()
-            images.toBlocks(blockSize, units=blockSizeUnits).saveAsBinarySeries(outputDirPath, overwrite=overwrite)
+        from thunder.rdds.fileio.imagesloader import ImagesLoader
+        loader = ImagesLoader(self._sc)
+        if inputFormat.lower() == 'stack':
+            images = loader.fromStack(dataPath, dims, ext=ext, dtype=dtype, startIdx=startIdx, stopIdx=stopIdx,
+                                      recursive=recursive, nplanes=nplanes, npartitions=npartitions,
+                                      confFilename=confFilename)
         else:
-            from thunder.rdds.fileio.seriesloader import SeriesLoader
-            if nplanes is not None:
-                raise NotImplementedError("nplanes is not supported with shuffle=False")
-            if npartitions is not None:
-                raise NotImplementedError("npartitions is not supported with shuffle=False")
-            loader = SeriesLoader(self._sc)
-            if inputFormat.lower() == 'stack':
-                loader.saveFromStack(dataPath, outputDirPath, dims, ext=ext, dtype=dtype,
-                                     blockSize=blockSize, overwrite=overwrite, startIdx=startIdx,
-                                     stopIdx=stopIdx, recursive=recursive)
-            else:
-                # 'tif' or 'tif-stack'
-                loader.saveFromTif(dataPath, outputDirPath, ext=ext, blockSize=blockSize,
-                                   startIdx=startIdx, stopIdx=stopIdx, overwrite=overwrite,
-                                   recursive=recursive)
+            # 'tif' or 'tif-stack'
+            images = loader.fromTif(dataPath, ext=ext, startIdx=startIdx, stopIdx=stopIdx,
+                                    recursive=recursive, nplanes=nplanes, npartitions=npartitions)
+        if renumber:
+            images = images.renumber()
+        images.toBlocks(blockSize, units=blockSizeUnits).saveAsBinarySeries(outputDirPath, overwrite=overwrite)
 
-    def makeExample(self, dataset, **opts):
+    def makeExample(self, dataset=None, **opts):
         """
         Make an example data set for testing analyses.
 
-        Options include 'pca', 'kmeans', and 'ica'.
+        Options include 'pca', 'factor', 'kmeans', 'ica', 'sources'
         See thunder.utils.datasets for detailed options.
 
         Parameters
@@ -520,7 +565,12 @@ class ThunderContext():
             Generated dataset
 
         """
-        checkParams(dataset, ['kmeans', 'pca', 'ica'])
+        from thunder.utils.datasets import DATASET_MAKERS
+
+        if dataset is None:
+            return sorted(DATASET_MAKERS.keys())
+
+        checkParams(dataset, DATASET_MAKERS.keys())
 
         return DataSets.make(self._sc, dataset, **opts)
 
@@ -562,7 +612,7 @@ class ThunderContext():
         checkParams(dataset, DATASETS.keys())
 
         if 'ec2' in self._sc.master:
-            tmpdir = os.path.join('/root/thunder/python/thunder/utils', 'data', DATASETS[dataset])
+            tmpdir = os.path.join('/root/thunder/thunder/utils', 'data', DATASETS[dataset])
         else:
             tmpdir = tempfile.mkdtemp()
             atexit.register(shutil.rmtree, tmpdir)
@@ -633,9 +683,35 @@ class ThunderContext():
 
         return data, params
 
+    def loadJSON(self, path):
+        """
+        Generic function for loading JSON from a path, handling local file systems and S3 or GS
+
+        Parameters
+        ----------
+        path : str
+            Path to a file, can be on a local file system or an S3 or GS bucket
+
+        Returns
+        -------
+        A string with the JSON
+        """
+
+        import json
+        from thunder.rdds.fileio.readers import getFileReaderForPath, FileNotFoundError
+        from thunder.utils.serializable import _decode_dict
+
+        reader = getFileReaderForPath(path)(awsCredentialsOverride=self._credentials)
+        try:
+            buffer = reader.read(path)
+        except FileNotFoundError:
+            raise Exception("Cannot find file %s" % path)
+
+        return json.loads(buffer, object_hook=_decode_dict)
+
     def loadParams(self, path):
         """
-        Load a file with parameters from a local file system or S3.
+        Load a file with parameters from a local file system or S3 or GS.
 
         Assumes file is JSON with basic types (strings, integers, doubles, lists),
         in either a single dict or list of dict-likes, and each dict has at least
@@ -646,29 +722,43 @@ class ThunderContext():
         Parameters
         ----------
         path : str
-            Path to file, can be on a local file system or an S3 bucket
+            Path to file, can be on a local file system or an S3 or GS bucket
 
         Returns
         -------
         A dict or list with the parameters
         """
-        import json
-        from thunder.rdds.fileio.readers import getFileReaderForPath, FileNotFoundError
+        blob = self.loadJSON(path)
+        return Params(blob)
 
-        reader = getFileReaderForPath(path)(awsCredentialsOverride=self._credentials)
-        try:
-            buffer = reader.read(path)
-        except FileNotFoundError:
-            raise Exception("Cannot find file %s" % path)
+    def loadSources(self, path):
+        """
+        Load a file with sources from a local file system or S3 or GS.
 
-        return Params(json.loads(buffer))
+        Parameters
+        ----------
+        path : str
+            Path to file, can be on a local file system or an S3 or GS bucket
+
+        Returns
+        -------
+        A SourceModel
+
+        See also
+        --------
+        SourceExtraction
+        """
+        from thunder import SourceExtraction
+
+        blob = self.loadJSON(path)
+        return SourceExtraction.deserialize(blob)
 
     def export(self, data, filename, outputFormat=None, overwrite=False, varname=None):
         """
         Export local array data to a variety of formats.
 
-        Can write to a local file sytem or S3 (destination inferred from filename schema).
-        S3 writing useful for persisting arrays when working in an environment without
+        Can write to a local file sytem or S3 or GS (destination inferred from filename schema).
+        S3 or GS writing useful for persisting arrays when working in an environment without
         accessible local storage.
 
         Parameters
@@ -719,23 +809,24 @@ class ThunderContext():
         """
         Manually set AWS access credentials to be used by Thunder.
 
-        Provided for hosted cloud environments without filesystem access.
-        If launching a cluster using the thunder-ec2 script, credentials will be configured
-        automatically (inside core-site.xml and ~/.boto), so this method should not need to be called.
+        Provided for hosted cloud environments without filesystem access. If
+        launching a cluster using the thunder-ec2 script, credentials will be
+        configured automatically (inside core-site.xml and ~/.boto), so this
+        method should not need to be called.
 
         Parameters
         ----------
         awsAccessKeyId : string
-             AWS public key, usually starts with "AKIA"
+            AWS public key, usually starts with "AKIA"
 
         awsSecretAccessKey : string
             AWS private key
+        
         """
-        from thunder.utils.common import AWSCredentials
+        from thunder.utils.aws import AWSCredentials
         self._credentials = AWSCredentials(awsAccessKeyId, awsSecretAccessKey)
         self._credentials.setOnContext(self._sc)
 
-
 DEFAULT_EXTENSIONS = {
     "stack": "bin",
     "tif": "tif",
